# Basic Defintions for using GNU-compiler suite sequentially
# requires setting of COMPILER=GCC_

#  Directories to search for
#    on mephisto
#CUDA_HOME = /usr/local/cuda-7.5
#SDK_HOME = /hdd/home/ghaase/NVIDIA_GPU_Computing_SDK/C/common
#BLAS = -I/share/apps/atlas/include  -L/share/apps/atlas/lib -lcblas -latlas
#
#    on Sony-Haase
#CUDA_HOME = /usr
#SDK_HOME = /home/ghaase/NVIDIA_GPU_Computing_SDK/C/common
#OPTIRUN = optirun               ##  now:  export OPTIRUN = "optirun -v"
#BLAS = -lcblas


#CC	= gcc
#CXX     = $(CUDA_HOME)/bin/nvcc
CXX     = nvcc
#F77	= gfortran
LINKER  = ${CXX}


#WARNINGS = -Wall -Weffc++ -Woverloaded-virtual -W -Wfloat-equal -Wshadow \
#           -Wredundant-decls -Winline
#  -Wunreachable-code
WARNINGS =  -src-in-ptx  
#WARNINGS +=  -sp-bound-check -warn-double-usage -warn-lmem-usage -warn-spills -res-usage 
#            |--CUDA 7.5   |-- slow !!
# CXXFLAGS += -DNDEBUG ${WARNINGS}

# CUDA
PERF = -use_fast_math -restrict
DEBUG = -G -lineinfo
# Ubuntu 16.04 with nvcc V7.5.17  cannot cope with -O1 and higher optimization levels
#CXXFLAGS += -O3  ${PERF} -arch sm_20 --ptxas-options=-v -lineinfo  ${WARNINGS} -I$(SDK_HOME)/inc
#OPT_GPU = -Xptxas --allow-expensive-optimizations
CXXFLAGS += -O0  ${PERF} -arch sm_20 --ptxas-options=-v ${OPT_GPU} ${WARNINGS} -I$(SDK_HOME)/inc
LINKFLAGS += -lm ${BLAS} -lcudart -lcublas

default:	${PROGRAM}

${PROGRAM}:	${OBJECTS}
	$(LINKER)  $^  ${LINKFLAGS} -o $@

clean::
	@rm -f ${PROGRAM} ${OBJECTS}

clean_all:: clean
	@rm -f *_ *~ *.bak *.log *.out *.tar

run: clean ${PROGRAM}
	${OPTIRUN} ./${PROGRAM}

# tar the current directory
MY_DIR = `basename ${PWD}`
tar: clean_all
	@echo "Tar the directory: " ${MY_DIR}
	@cd .. ;\
	tar cf ${MY_DIR}.tar ${MY_DIR} *default.mk ;\
	cd ${MY_DIR}
# 	tar cf `basename ${PWD}`.tar *

doc:
	doxygen Doxyfile

#########################################################################
.PRECIOUS: .cu .h
.SUFFIXES: .cu .h .o

.cu.o:
	$(CXX) -c $(CXXFLAGS) $<

.cpp.o:
	$(CXX) -c $(CXXFLAGS) $<

.c.o:
	$(CC) -c $(CFLAGS) $<

.f.o:
	$(F77) -c $(FFLAGS) $<

##################################################################################################

# Check for wrong memory accesses, memory leaks, ...
# use smaller data sets
#  CXXFLAGS += -g -G
#  LINKFLAGS += -pg
cache: ${PROGRAM}
	${OPTIRUN} nvprof --print-gpu-trace  ./$^ > out_prof.txt
	#${OPTIRUN} nvprof --events l1_global_load_miss,l1_local_load_miss  ./$^ > out_prof.txt

mem: ${PROGRAM}
	${OPTIRUN} cuda-memcheck ./$^

#  Simple run time profiling of your code
#  CXXFLAGS  += -g -G -lineinfo
#  LINKFLAGS += -g -G -lineinfo
prof: ${PROGRAM}
	${OPTIRUN} ./$^
	${OPTIRUN} nvvp ./$^ &
