#include <iostream>
#include <cmath>
#include <omp.h>                // OpenMP
#ifdef _OPENACC
#include <accel.h>              // OpenACC
#endif
using namespace std;
#include "vdop.h"
#include "getmatrix.h"
#include "jacsolve.h"

void JacobiSolve(const int nnode,
// 	const int neigh[], const int color,
// 	const MPI::Intracomm& icomm,
                 const int id[], const int ik[], const float sk[],
                 const float f[], float u[] )
{
    const float omega   = 1.0;
    const int    maxiter = 1000;
    const float tol  = 1e-5,                  /* tolerance   */
                 tol2 = tol * tol;             /* tolerance^2 */

    float *dd = new float [nnode],           /* matrix diagonal */
           *r  = new float [nnode],           /* residual */
           *w  = new float [nnode];           /* correction */

    #pragma acc data copy(u[0:nnode]) copyin(f[0:nnode],id[0:nnode+1],ik[0:id[nnode]],sk[0:id[nnode]]) \
    create(dd[0:nnode],r[0:nnode],w[0:nnode])
   {

        {
            cout << endl << " Start Jacobi solver for " << nnode << " d.o.f.s"  << endl;
        }
        float sigma0,sigma;
        int iter;
        //#pragma acc parallel
        {
        /*  Choose initial guess  */
        #pragma omp parallel for
        #pragma acc kernels pcopy(u[0:nnode])
        for (int k=0; k<nnode; ++k) { u[k] = 0.0; }  /*  u := 0         */

        GetDiag(nnode, id, ik, sk, dd);            /*  diag */
           //DebugVector(nnode,dd);{int ijk; cin >> ijk;}    // Debug values on Host
                                                   /*  Initial sweep  */
        Defect(r, f, u, nnode, id, ik, sk);        /*  r := f - K*u   */

        //#pragma acc kernels loop independent pcopy(w[0:nnode]) pcopyin(r[0:nnode], dd[0:nnode])
        //for (int k=0; k<nnode; ++k)
         //{
			 //#pragma atomic update
			 //w[k % 7] += fabsf(r[k])/dd[k];
		 //}

        #pragma omp parallel for
        #pragma acc kernels loop independent pcopyout(w[0:nnode]) pcopyin(r[0:nnode], dd[0:nnode])
        for (int k=0; k<nnode; ++k) { w[k] = r[k]/dd[k]; }  /*  w := D^{-1}*r  */

        iter  = 0;
        sigma0 = dscapr(nnode, w, r);              /* s0 := <w,r>     */
        sigma  = sigma0;
        //cout << "f0: "; DebugVector(nnode,f);/*{int ijk; cin >> ijk;} */
        //cout << "r0: "; DebugVector(nnode,r);/*{int ijk; cin >> ijk;} */
        //cout << "sk: "; DebugVector(id[nnode],sk);/*{int ijk; cin >> ijk;} */
        //cout << "dd: "; DebugVector(nnode,dd);/*{int ijk; cin >> ijk;} */
        //cout << "w0: "; DebugVector(nnode,w);/*{int ijk; cin >> ijk;} */

        /* Iteration sweeps */
        while ( sigma > tol2 * sigma0 && maxiter > iter) {
            ++iter;
            vdaxpy(nnode, u, u, omega, w );          /*  u := u + om*w  */
            Defect(r, f, u, nnode, id, ik, sk);      /*  r := f - K*u   */
               //cout << "r: "; DebugVector(nnode,r);/*{int ijk; cin >> ijk;} */   // Debug values on Host

            #pragma omp parallel for
            #pragma acc kernels loop independent pcopyout(w[0:nnode]) pcopyin(r[0:nnode], dd[0:nnode])
            for (int k=0; k<nnode; ++k) { w[k] = r[k]/dd[k]; }  /*  w := D^{-1}*r  */

            sigma = dscapr(nnode, w, r);             /* s0 := <w,r>     */
      	    //if ( omp_get_thread_num()==0) cout << "Iteration " << iter << " : " << sqrt(sigma/sigma0) << endl;
            #pragma omp barrier
        }

	} // end: acc parallel
        {
           cout << "aver. Jacobi rate :  " << exp(log(sqrt(sigma/sigma0))/iter) << "  (" << iter << " iter)"<< endl;
           cout << "final error: " << sqrt(sigma / sigma0) << " (rel)   " << sqrt(sigma) << " (abs)\n";
        }
    } // end: acc data

    //#pragma acc exit data copyout(u[0:nnode])    // here, not needed

    delete [] w;
    delete [] r;
    delete [] dd;
    return;
}

