#ifndef USERSET_FILE
#define USERSET_FILE


/**
 * User function: assignes values to solution vector @p u.
 * @param[in] nx	number of discretization intervals in x-direction
 * @param[in] ny	number of discretization intervals in y-direction
 * @param[out] u	vector for all nodes of local discretization
*/
void SetU(const int nx, const int ny, float u[]);

/**
 * User function: assignes values to right hand side vector @p f.
 * @param[in] nx	number of discretization intervals in x-direction
 * @param[in] ny	number of discretization intervals in y-direction
 * @param[out] f	vector for all nodes of local discretization
*/
void SetF(const int nx, const int ny, float f[]);

/**
 * User function: f(@p x,@p y)
 * @param[in] x		x-coordinate of discretization point
 * @param[in] y		y-coordinate of discretization point
 * @return  value for right hand side f(@p x,@p y)
 */
//#pragma acc routine seq
float FunctF(const float& x, const float& y);

/**
 * User function: u(@p x,@p y)
 * @param[in] x		x-coordinate of discretization point
 * @param[in] y		y-coordinate of discretization point
 * @return  value for solution vector u(@p x,@p y)
 */
float FunctU(const float& x, const float& y);

#endif
 
