
#include <iostream>
#include <cmath>
using namespace std;

//----------------------------------------------------------------------
//    from  CUDA/skalar/skalar_3.cu
//
//    kernel function on device
__global__ void scalar(float *sg, const float *x, const float *y, int N)
{
    extern __shared__ float sdata[];
    const int idx = blockIdx.x * blockDim.x + threadIdx.x;
    const int str = gridDim.x * blockDim.x;

    float ss = 0.0;
    for (int i = idx; i < N; i += str)
        ss += x[i] * y[i];
    sdata[threadIdx.x] = ss;

    __syncthreads();
    for (int s = blockDim.x >> 1; s > 0; s >>= 1) {
        if (threadIdx.x < s) {
            sdata[threadIdx.x] += sdata[threadIdx.x + s];
        }
        __syncthreads();
    }
    if (threadIdx.x == 0)  sg[blockIdx.x] = sdata[0];
}


//    kernel function for addition on  o n e  block on device
__global__ void add_1_block(float *s, int N)
{
    if (N > blockDim.x) return;
    extern __shared__ float sdata[];
    const int tid = threadIdx.x;
    if (tid < N) sdata[tid] = s[tid];
    else       sdata[tid] = 0.0;

    __syncthreads();
    for (int s = blockDim.x / 2; s > 0; s >>= 1) {
        if (tid < s) {
            sdata[tid] += sdata[tid + s];
        }
        __syncthreads();
    }
    if (tid == 0)  s[0] = sdata[0];
}
//--------------------------------------------------------------------
//float dscapr_GPU_old(const int n, const float x[], const float y[])
//{
    //cudaDeviceProp prop;
    //cudaGetDeviceProperties (&prop, 0);
    //const int blocksize = 4 * 64,
////           gridsize = ceil( N / static_cast<float>(blocksize)),
              //gridsize = prop.multiProcessorCount,
              //resBytes = gridsize * sizeof(float),
              //sBytes   = blocksize * sizeof(float) ;
    //dim3 dimBlock(blocksize);
    //dim3 dimGrid(gridsize);
//// allocate memory for temp. vector on device
    //float *s_d;               // single number
    //cudaMalloc((void **) &s_d, resBytes);


//// call the kernel function with  dimGrid.x * dimBlock.x threads
    //scalar <<< dimGrid, dimBlock, sBytes>>>(s_d, x, y, n);
////    power of 2 as number of treads per block
    //const unsigned int oneBlock = 2 << (int)ceil(log(dimGrid.x + 0.0) / log(2.0));
    ////cudaThreadSynchronize();
    //add_1_block <<< 1, oneBlock, oneBlock *sizeof(float)>>>(s_d, dimGrid.x);
//// copy data:  device --> host
    //float sum;
    //cudaMemcpy(&sum, s_d, sizeof(float), cudaMemcpyDeviceToHost);

    //cudaFree(s_d);
    //return sum;
//}

//----------------------------------------------------------------------
static bool b_init=false;

/** 	Host function. Inner product calculated with device vectors.
 *  The implementation consists of two kernel with a temporary array for data transfer between them.
 *  The necessary dynamic memory alocation is done only in the first function call (global variable @p b_init).
 *  This  dynamic memory is not deallocated afterwards.
	@param[in] N	number of vector elements
	@param[in] x_d	vector on device
	@param[in] y_d	vector on device
	@return 	resulting Euclidian inner product <x_d,y_d> on host
	@warning    The dynamic memory is not deallocated afterwards.
*/
float dscapr_GPU(const int N, const float x_d[], const float y_d[])
{
	static float *s_d;
	static dim3 dimBlock, dimGrid;
	static int sBytes;
	if (!b_init)   // only  in the first function call
	{              //   get device properties and allocate memory
		b_init = !b_init;
		cudaDeviceProp prop;
        cudaGetDeviceProperties (&prop, 0);

        const int blocksize = 8 * 64,
                  gridsize = prop.multiProcessorCount;
        sBytes   = blocksize * sizeof(float);
        dimBlock = dim3(blocksize);
        dimGrid  = dim3(4*gridsize);

        //float s_d;
        cudaMalloc((void **) &s_d, blocksize * sizeof(float)); // temp. memory on device
        cout << endl << gridsize << " x " << blocksize << " Threads\n";
    }

// call the kernel function with  dimGrid.x * dimBlock.x threads
    scalar <<< dimGrid, dimBlock, sBytes>>>(s_d, x_d, y_d, N);
//    power of 2 as number of treads per block
    const unsigned int oneBlock = 2 << (int)ceil(log(dimGrid.x + 0.0) / log(2.0));
    add_1_block <<< 1, oneBlock, oneBlock *sizeof(float)>>>(s_d, dimGrid.x);
// copy data:  device --> host
    float sg;
    cudaMemcpy(&sg, s_d, sizeof(float), cudaMemcpyDeviceToHost);

    //cudaFree(s_d);                   // GH: ToDo: Free memory after the end of all computations
    return sg;
}
