	  // scalar product: 5: use of thrust 
	  //
	  //  CUDA: nvcc -O3  -use_fast_math -restrict -arch sm_20 -lm  -lcudart -lcublas  skalar_5.cu
	  //
	  //  x86, OpenMP:   nvcc -O3 -Xcompiler -fopenmp -DTHRUST_DEVICE_SYSTEM=THRUST_DEVICE_BACKEND_OMP -lgomp skalar_5.cu 
	  //        !!!  OpenMP requires omp_get_wtime()  for timing to measure WTime !!!

	  
	  #include <cassert>
	  #include <iostream>
	  #include <ctime>
	  using namespace std;
	  #include <thrust/host_vector.h>
	  #include <thrust/device_vector.h>
	  #include <thrust/inner_product.h>
	  //  main function on host
	  int main(void)
	  {
	  cudaDeviceProp prop;
	  cudaGetDeviceProperties (&prop, 0);
	  cout<< endl << "We work on a "<< prop.name << " GPU  with " << 
		prop.multiProcessorCount << " Multiprocessors"<< endl;
		    
	  double sum, tstart, tgpu;
	  const int N = 14000000;
	  const int LOOP = 100;
	  
	  thrust::host_vector<double> x_h(N);
	  thrust::host_vector<double> y_h(N);
	  
	  double ss=0.0;
	  for (int i=0; i<N; i++) 
	    { x_h[i] = (i % 137)+1; y_h[i] = 1.0/x_h[i]; ss+=x_h[i]*y_h[i];}
	    
	  // allocate memory on device and copy dato into it
	  thrust::device_vector<double> x_d = x_h;
	  thrust::device_vector<double> y_d = y_h;

	  // thrust::plus<float>       binary_op1;
	  // thrust::multiplies<float> binary_op2;

	  tstart = clock();
	  for (int k=0; k<LOOP; ++k) {
	    sum = thrust::inner_product(x_d.begin(), x_d.end(), y_d.begin(), 0.0);
	    // This is exactly the same as:
	    // sum = thrust::inner_product(x_d.begin(), x_d.end(), y_d.begin(), 0.0, binary_op1, binary_op2);
	  }
	  tgpu = (clock() - tstart)/CLOCKS_PER_SEC/LOOP;
	  
	  cout << "\nN = " << N << endl;
	  cout << "\nGPU: " << tgpu << " sec.; " << 
	    2e-9*N/tgpu << " GFLOPS;  " << 
	    2.0*N*sizeof(double)/tgpu/1024/1024/1024 << " GiB/sec.\n ";

	  if (static_cast<int>(sum)!=N)
	      {
	      cout << " !!  W R O N G  result  " << endl;
	      cout << sum << "  vs.  " << N << "(" << ss<< ")" << endl;
	      }
	  else
	      cout << " correct  result  " << endl;

	  return 0;
	  }
