//		MPI code in C++.
//		See [Gropp/Lusk/Skjellum, "Using MPI", p.33/41 etc.]
//		and  /opt/mpich/include/mpi2c++/comm.h  for details
//  Compile with
//   make 2>&1 | grep -v openmpi
//  to avoid warnings caused by OpenMPI
#include <iostream>
#include <cmath>
using namespace std;
#include <mpi.h>
#include "debugd.h"
#include "geom.h"
#include "vecaccu.h"

int main(int argc,char* argv[])
{
    MPI_Comm icomm = MPI_COMM_WORLD;
    MPI_Init(&argc, &argv);
    int myrank, numprocs;
    MPI_Comm_rank(icomm, &myrank);                  // my MPI-rank
    MPI_Comm_size(icomm, &numprocs);

    if (myrank == 0)
    { cout << "\n There are " << numprocs << " processes running.\n \n"; }

    MPI_Barrier(icomm);

//     procx = (int) sqrt((double) numprocs);
//     procy = (int) sqrt((double) numprocs);
    const int procx = static_cast<int>(sqrt(numprocs+0.0));
    const int procy = procx;

    if (procy * procx != numprocs)
    {
      cout << "\n Wrong number of processors !\n \n";
    }
    else
    {
 //		initialize domain decomposition
      int neigh[4], color;
      IniGeom(myrank, procx, procy, neigh, color);

      MPI_Barrier(icomm);
//		get subdomain coordinates
      double xl, xr, yb, yt;
      IniCoord(myrank, procx, procy, xl, xr, yb, yt);

      /* number of intervals on (xl,xr)=:nx, (yb,yt)=:ny */
      const int nx = 5;
      const int ny = 6;
      /* number of nodes per direction (xl,xr)=nx+1=:nex, (yb,yt)=ny+1=:ney */
      const int nnx = nx + 1;
      const int nny = ny + 1;
      /* number of degrees of freedom  */
      const int nd = nnx*nny;

      /* allocate memory */
      double *u = new double [nd];

      /* initialize u */
      for (int i=0; i<nd; ++i)
      	   u[i] = myrank*100.0+i;

      DebugD(nd,u);

      VecAccu(nx,ny,u,neigh,color,icomm);

      DebugD(nd,u);

      delete [] u;
      FreeVecAccu();
  }

  MPI_Finalize();
}


