#include <iostream>
#include <cstdlib>
using namespace std;
#include <mpi.h>

const int TAG1 = 1;
const int TAG2 = 2;

void Send_ProcD(const int to, const int nin, const double xin[], const MPI_Comm icomm)
{
//   cout << icomm.Get_rank() << "  -->  " << to << "  (length " << nin << ")\n";

    //MPI_Send(&nin, 1, MPI_INT, to, TAG1, icomm);
    ////icomm.Ssend(&nin, 1, MPI::INT, to, TAG1);
    MPI_Request rq;
    MPI_Isend(&nin, 1, MPI::INT, to, TAG1, icomm, &rq);

//   int info=stat.Get_error();
//   if (info!=MPI::SUCCESS)
//   { cout << "\n Send_ProcD : " << info << endl; exit(-3); }

    if (nin > 0) {
        //MPI_Send(const_cast<double*>(xin), nin, MPI_DOUBLE, to, TAG2, icomm);
        ////icomm.Ssend(xin, nin, MPI::DOUBLE, to, TAG2);
        MPI_Isend(xin, nin, MPI::DOUBLE, to, TAG2, icomm, &rq);
    }

    return;
}



void Recv_ProcD(const int from, int &nout, double xout[], const int maxbuf,
                const MPI::Intracomm &icomm)
{
    MPI_Status stat;
    stat.MPI_ERROR = 0;            // M U S T   be initialized!!
    MPI_Request rq;

    int ierror = MPI_Irecv(&nout, 1, MPI_INT, from, TAG1, icomm, &rq);

    int info = stat.MPI_ERROR;          // C++: int info=stat.Get_error();

    ierror = MPI_Waitall(1, &rq, &stat);

//    http://stackoverflow.com/questions/4831628/unknown-field-in-mpi-status
    if (stat.MPI_ERROR != MPI_SUCCESS && ierror) {
        cout << "\n Recv_ProcD : " << info << endl;
        exit(-3);
    }

    if (nout > maxbuf) {
        cout << "\n Recv_ProcD : maxbuf (= " << maxbuf << ") is too small vs. "
             << nout << endl;
        exit(-3);
    }

    if (nout > 0) {
        MPI_Irecv(xout, nout, MPI_DOUBLE, from, TAG2, icomm, &rq);
        ierror = MPI_Waitall(1, &rq, &stat);
    }

    return;
}


void ExchangeD_org(const int yourid, const int nin, const double xin[],
                   int &nout, double xout[], const int maxbuf, const MPI_Comm icomm)
{
    int myrank;
    MPI_Comm_rank(icomm, &myrank);

    if (yourid < 0) {
        nout = -1;
        return;
    }

    if (myrank < yourid) {
        Send_ProcD(yourid, nin, xin, icomm);
        Recv_ProcD(yourid, nout, xout, maxbuf, icomm);
    }
    else {
        Recv_ProcD(yourid, nout, xout, maxbuf, icomm);
        Send_ProcD(yourid, nin, xin, icomm);
    }

    return;
}


// No check for message lenght anymore
void ExchangeD(const int yourid, const int nin, const double xin[],
               int &nout, double xout[], const int , const MPI_Comm icomm)
{
    if (yourid < 0) {
        nout = -1;
        return;
    }

    MPI_Status stat;
    stat.MPI_ERROR = 0;            // M U S T   be initialized!!
    MPI_Request rq, dummy;

    MPI_Irecv(xout, nin, MPI_DOUBLE, yourid, TAG2, icomm, &rq);
    MPI_Isend(xin, nin, MPI::DOUBLE, yourid, TAG2, icomm, &dummy);
    nout = nin;

    //int myrank;
    //MPI_Comm_rank(icomm, &myrank);
    //cout << myrank << "  -->  " << yourid << "  (length " << nin << ")\n";

    MPI_Waitall(1, &rq, &stat);      // MPI_Wait ??

    return;
}

//  http://mpi.deino.net/mpi_functions/MPI_Wait.html
//  http://mpi.deino.net/mpi_functions/index.htm
//  https://svn.mcs.anl.gov/repos/mpi/test_suites/intel/trunk/Test/c/blocking/error/MPI_Send_err1/node.c
