#include <iostream>
using namespace std;
#include <mpi.h>
#include "dexx.h"
#include "geom.h"

const int TAG2 = 2;

void VecAccu(const int nx, const int ny, double w[],
             const int neigh[], const int , const MPI_Comm icomm)
{
    static int count = 0, lnn;
    static double *v_in[4], *v_out[4];
    static const int nnn[4] = {nx + 1, ny + 1, nx + 1, ny + 1};

//		manage memory allocation for the first function call
    if (count == 0) {
        lnn = max(nx, ny) + 1;
        for (int dir = 0; dir < 4; ++dir) {
            v_in[dir]  = new double [lnn];
            v_out[dir] = new double [lnn];
        }
    }
    else {
        if ( nx + 1 > lnn || ny + 1 > lnn ) {	// memory reallocation necessary?
            lnn = max(nx, ny) + 1;
            for (int dir = 0; dir < 4; ++dir) {
                delete [] v_in[dir];
                delete [] v_out[dir];
                v_in[dir]  = new double [lnn];
                v_out[dir] = new double [lnn];
            }
        }
    }
//
// I have to split the communication into the south-north part followed
// by the west-eats part.
// Otherwise the crosspoints are not handled correctly,
// crosspoint on P_0 doesn't get the data from P_3 in case of a 2x2 decomposition.
//
    for (int k = 0; k < 2; ++k) { // k==0: South <--> North,  k==1: East <--> West
        MPI_Request rq[2];

        int cnt = 0;
        for (int dir = k; dir < 4; dir += 2) {
            if (neigh[dir] >= 0) {
                GetBound(dir + 1, nx, ny, w, v_in[dir]);

                MPI_Request dummy;
                MPI_Irecv(v_out[dir], nnn[dir], MPI_DOUBLE, neigh[dir], TAG2, icomm, &rq[cnt]);
                MPI_Isend(v_in[dir] , nnn[dir], MPI_DOUBLE, neigh[dir], TAG2, icomm, &dummy);
                //  http://stackoverflow.com/questions/10882581/mpi-isend-request-parameter
                MPI_Request_free(&dummy);
                ++cnt;
            }
        }

        MPI_Waitall(cnt, rq, MPI_STATUSES_IGNORE);

        for (int dir = k; dir < 4; dir += 2) {
            if (neigh[dir] >= 0) {
                AddBound(dir + 1, nx, ny, w, v_out[dir]);
            }
        }
    }
    return;
}

void FreeVecAccu()
{
    static double *v_in[4], *v_out[4];

    for (int dir = 0; dir < 4; ++dir) {
        delete [] v_in[dir];
        delete [] v_out[dir];
    }

    return;
}


//----------------------------------------------------------------------
// Original part
//----------------------------------------------------------------------

void VecAccu_old(const int nx, const int ny, double w[],
                 const int neigh[], const int color, const MPI_Comm icomm)
{
    static int count = 0, lnx, lny;
    static double *South_in, *East_in, *North_in, *West_in,
           *South_out, *East_out, *North_out, *West_out;

    const int nnx = nx + 1;
    const int nny = ny + 1;

//		manage memory allocation for the first function call
    if (count == 0) {
        South_in = new double [nnx];
        East_in  = new double [nny];
        North_in = new double [nnx];
        West_in  = new double [nny];
        South_out = new double [nnx];
        East_out  = new double [nny];
        North_out = new double [nnx];
        West_out  = new double [nny];
        lnx = nnx;
        lny = nny;
        count = 1;
    }
    else {
        if (nnx > lnx) {	// memory reallocation necessary?
            delete [] South_in;
            South_in  = new double [nnx];
            delete [] North_in;
            North_in  = new double [nnx];
            delete [] South_out;
            South_out = new double [nnx];
            delete [] North_out;
            North_out = new double [nnx];
            lnx = nnx;
        }
        if (nny > lny) {	// memory reallocation necessary?
            delete [] East_in;
            East_in  = new double [nny];
            delete [] West_in;
            West_in  = new double [nny];
            delete [] East_out;
            East_out = new double [nny];
            delete [] West_out;
            West_out = new double [nny];
            lny = nny;
        }
    }

    int nout[4];
    int nin = nnx;
    GetBound(1, nx, ny, w, South_in);
    GetBound(3, nx, ny, w, North_in);

    if (color == 0) {
        ExchangeD(neigh[0], nin, South_in, nout[0], South_out, nin, icomm);
        ExchangeD(neigh[2], nin, North_in, nout[2], North_out, nin, icomm);
    }
    else {
        ExchangeD(neigh[2], nin, North_in, nout[2], North_out, nin, icomm);
        ExchangeD(neigh[0], nin, South_in, nout[0], South_out, nin, icomm);
    }

    if (neigh[0] >= 0)  AddBound(1, nx, ny, w, South_out);
    if (neigh[2] >= 0)  AddBound(3, nx, ny, w, North_out);

    nin =  nny;
    GetBound(2, nx, ny, w, East_in);
    GetBound(4, nx, ny, w, West_in);


    if (color == 0) {
        ExchangeD(neigh[1], nin, East_in, nout[1], East_out, nin, icomm);
        ExchangeD(neigh[3], nin, West_in, nout[3], West_out, nin, icomm);
    }
    else {
        ExchangeD(neigh[3], nin, West_in, nout[3], West_out, nin, icomm);
        ExchangeD(neigh[1], nin, East_in, nout[1], East_out, nin, icomm);
    }

    if (neigh[1] >= 0)   AddBound(2, nx, ny, w, East_out);
    if (neigh[3] >= 0)   AddBound(4, nx, ny, w, West_out);

    return;
}


void FreeVecAccu_old()
{
    static double *South_in, *East_in, *North_in, *West_in,
           *South_out, *East_out, *North_out, *West_out;

    delete [] South_in;
    delete [] East_in;
    delete [] North_in;
    delete [] West_in;
    delete [] South_out;
    delete [] East_out;
    delete [] North_out;
    delete [] West_out;
    return;
}
