//		MPI code in C++.
//		See [Gropp/Lusk/Skjellum, "Using MPI", p.33/41 etc.]
//		and  /opt/mpich/include/mpi2c++/comm.h  for details
//  Compile with
//   make 2>&1 | grep -v openmpi
//  to avoid warnings caused by OpenMPI
#include <iostream>
#include <cmath>
using namespace std;
#include <mpi.h>
#include "debugd.h"
#include "geom.h"
#include "vecaccu.h"
#include "skalar.h"

#include "communicator.h"

int main(int argc, char *argv[])
{
    MPI::Init(argc, argv);
    MPI::Intracomm icomm(MPI::COMM_WORLD);
    int numprocs = icomm.Get_size();
    //int myrank   = icomm.Get_rank();

    //if (myrank == 0) {
        //cout << "\n There are " << numprocs << " processes running.\n \n";
    //}

    icomm.Barrier();

//     procx = (int) sqrt((double) numprocs);
//     procy = (int) sqrt((double) numprocs);
    const int procx = static_cast<int>(sqrt(numprocs + 0.0));
    const int procy = procx;

    if (procy * procx != numprocs) {
        cout << "\n Wrong number of processors !\n \n";
        return -1;
    }
    /* number of intervals on (xl,xr)=:nx, (yb,yt)=:ny */
    const int nx = 4;
    const int ny = 5;
    //{
////		initialize domain decomposition
        //int neigh[4], color;
        //IniGeom(myrank, procx, procy, neigh, color);

        //icomm.Barrier();
////		get subdomain coordinates
        //double xl, xr, yb, yt;
        //IniCoord(myrank, procx, procy, xl, xr, yb, yt);


        ///* number of nodes per direction (xl,xr)=nx+1=:nex, (yb,yt)=ny+1=:ney */
        //const int nnx = nx + 1;
        //const int nny = ny + 1;
        ///* number of degrees of freedom  */
        //const int nd = nnx * nny;

        ///* allocate memory */
        //double *u = new double [nd];

        ///* initialize u */
        //for (int i = 0; i < nd; ++i)
            //u[i] = myrank * 100.0 + i;

        ////DebugD(nd,u);

        //VecAccu(nx, ny, u, neigh, color, icomm);

        ////DebugD(nd,u);

        //double ska = skalar(nd, u, u, icomm);
        //if (myrank == 0) {
            //cout.precision(12);
            //cout << ska << endl << endl;
            ////if (1195170506 != static_cast<int>(ska))         // 4x4 domains
            ////if (177234285 != static_cast<int>(ska))          // 3x3 domains
            //if (9849768 != static_cast<int>(ska))            // 2x2 domains
                //cout << "\n W R O N G !\n\n";
        //}

        //delete [] u;
        //FreeVecAccu();
    //}
    //----------------------------------------------------------------
    {
        //Communicator mcomm(MPI::COMM_WORLD);
        Grid2D_Communicator jcomm(MPI::COMM_WORLD, procx, procy, nx, ny);
        int jmyrank   = jcomm.Get_rank();
        int jnumprocs = jcomm.Get_size();

        if (jmyrank == 0) {
            cout << "\n There are " << jnumprocs << " processes running.\n \n";
        }

        /* number of degrees of freedom  */
        const int nd = (nx+1) * (ny+1);
        /* allocate memory */
        double *u = new double [nd];

        /* initialize u */
        for (int i = 0; i < nd; ++i)
            u[i] = jmyrank * 100.0 + i;

        //DebugD(nd,u);

        //VecAccu(nx, ny, u, neigh, color, icomm);
        jcomm.VecAccu(nd, u);

        //DebugD(nd,u);

        //double ska = skalar(nd, u, u, icomm);
        double ska = jcomm.skalar(nd, u, u);
        if (jmyrank == 0) {
            cout.precision(12);
            cout << ska << endl << endl;
            //if (1195170506 != static_cast<int>(ska))         // 4x4 domains
            //if (177234285 != static_cast<int>(ska))          // 3x3 domains
            if (9849768 != static_cast<int>(ska))            // 2x2 domains
                cout << "\n W R O N G !\n\n";
        }
        jcomm.Get_comm().Barrier();

        // -------------------------------------------------------------
        Grid2D_gen_Communicator gcomm(MPI::COMM_WORLD, procx, procy, nx, ny);
        int grank   = gcomm.Get_rank();
        //int gnumprocs = gcomm.Get_size();
        if (grank == 0) cout << "\n Grid2D_gen_Communicator\n\n";

        /* initialize u */
        for (int i = 0; i < nd; ++i)
            u[i] = grank * 100.0 + i;

        gcomm.VecAccu(nd, u);
        //gcomm.Init_VecAccu(nd, u);
        //gcomm.Finish_VecAccu(nd, u);
        
        //DebugD(nd,u);

        ska = gcomm.skalar(nd, u, u);
        if (grank == 0) {
            cout.precision(12);
            cout << ska << endl << endl;
            //if (1195170506 != static_cast<int>(ska))         // 4x4 domains
            //if (177234285 != static_cast<int>(ska))          // 3x3 domains
            if (9849768 != static_cast<int>(ska))            // 2x2 domains
                cout << "\n W R O N G !\n\n";
            cout << "\n ##############################################\n";
       }
        gcomm.Get_comm().Barrier();


        delete [] u;
    }


    MPI_Finalize();
}


