#include <iostream>
using namespace std;
#include <mpi.h>
#include "dexx.h"
#include "geom.h"

const int TAG2 = 2;

// no check for message size anymore
void VecAccu(const int nx, const int ny, double w[],
             const int neigh[], const int , const MPI::Intracomm &icomm)
{
    static int count = 0, lnn;
    static double *v_in[4], *v_out[4];
    static const int nnn[4] = {nx + 1, ny + 1, nx + 1, ny + 1};

//		manage memory allocation for the first function call
    if (count == 0) {
        lnn = max(nx, ny) + 1;
        for (int dir = 0; dir < 4; ++dir) {
            v_in[dir]  = new double [lnn];
            v_out[dir] = new double [lnn];
        }
    }
    else {
        if ( nx + 1 > lnn || ny + 1 > lnn ) {	// memory reallocation necessary?
            lnn = max(nx, ny) + 1;
            for (int dir = 0; dir < 4; ++dir) {
                v_in[dir]  = new double [lnn];
                v_out[dir] = new double [lnn];
            }
        }
    }
//
// I have to split the communication into the south-north part followed
// by the west-eats part.
// Otherwise the crosspoints are not handled correctly,
// crosspoint on P_0 doesn't get the data from P_3 in case of a 2x2 decomposition.
//
    for (int k = 0; k < 2; ++k) { // k==0: South <--> North,  k==1: East <--> West
        MPI::Request rq[2];

        for (int dir = k; dir < 4; dir += 2) {
            if (neigh[dir] >= 0) {
                GetBound(dir + 1, nx, ny, w, v_in[dir]);
                rq[dir/2] = icomm.Irecv(v_out[dir], nnn[dir], MPI::DOUBLE, neigh[dir], TAG2);
                icomm.Isend(v_in[dir] , nnn[dir], MPI::DOUBLE, neigh[dir], TAG2);
            }
        }

        MPI::Request::Waitall(2, rq);

        for (int dir = k; dir < 4; dir += 2) {
            if (neigh[dir] >= 0) {
                AddBound(dir + 1, nx, ny, w, v_out[dir]);
            }
        }
    }
    return;
}


void FreeVecAccu()
{
    static double *v_in[4], *v_out[4];

    for (int dir = 0; dir < 4; ++dir) {
            delete [] v_in[dir];
            delete [] v_out[dir];
        }

    return;
}
