//		MPI code in C++.
//		See [Gropp/Lusk/Skjellum, "Using MPI", p.33/41 etc.]
//		and  /opt/mpich/include/mpi2c++/comm.h  for details

#include <iostream>
using namespace std;
#include <mpi.h>
#include "greetings.h"

int main(int argc, char *argv[])
{
	cout << "#######  " <<  argv[0]  << "  ##########\n";
    MPI_Comm icomm = MPI_COMM_WORLD;
    int myid, numprocs;
    MPI_Init(&argc, &argv);
    MPI_Comm_rank(icomm, &myid);                          // my MPI-rank
    MPI_Comm_size(icomm, &numprocs);

    if (myid == 0) {
        cout << "\n There are " << numprocs << " processes running.\n \n";
    }
// Spawn processes
// http://stackoverflow.com/questions/9707048/create-mpi-processes-on-the-fly-with-fork
// https://www.open-mpi.org/doc/v1.8/man3/MPI_Comm_spawn.3.php

    MPI_Comm parentcomm, intracomm;
    MPI_Comm_get_parent( &parentcomm );
    if (parentcomm == MPI_COMM_NULL)
    {
		MPI_Comm intercomm;
		char ss[]="./main.GCC_";
		//char ss[]="./old_main";        //  requires:  cp  main.GCC_  old_main
        MPI_Comm_spawn( ss, MPI_ARGV_NULL, 3, MPI_INFO_NULL, 0, MPI_COMM_WORLD, &intercomm, MPI_ERRCODES_IGNORE);
        // http://stackoverflow.com/questions/9970409/mpi-spawn-root-process-does-not-communicate-to-child-processes
        // intracommunicator = intercommunicator + current intracommunicator
        MPI_Intercomm_merge(intercomm, 0, &intracomm);
		MPI_Comm_rank(intracomm, &myid);                          // my MPI-rank
        MPI_Comm_size(intracomm, &numprocs);
        cout << "parent : " <<  myid << "  " << numprocs << endl;
	}
	else
	{
        // http://stackoverflow.com/questions/9970409/mpi-spawn-root-process-does-not-communicate-to-child-processes
		// intracommunicator = parentcommunicator + current intracommunicator
		MPI_Intercomm_merge(parentcomm, 1, &intracomm);
		MPI_Comm_rank(intracomm, &myid);                          // my MPI-rank
        MPI_Comm_size(intracomm, &numprocs);
        cout << "spawned : " <<  myid << "  " << numprocs << endl;
	}
//
    MPI_Barrier(intracomm);    // intracomm instead of icomm

    greetings(intracomm);      // intracomm instead of icomm

    if (myid == 0) cout << endl;

// deactivate the following line when the parent process is started without mpirun
    MPI_Finalize();

    return 0;
}


