#ifndef HEADER_ACCUDIAG
#define HEADER_ACCUDIAG

/**
 * Copies the diagonal entries of a distributed symmetric CSR matrix (@p sk,@p id, @p ik) 
 * onto a vector and accumulates them.
 * @param[in] nx	number of discretization intervals in x-direction
 * @param[in] ny	number of discretization intervals in y-direction
 * @param[in] sk	vector non-zero entries of CSR matrix
 * @param[in] id	index vector containing the first entry in a CSR row
 * @param[out] d	accumulated diagonal entries of CSR matrix
 * @param[in] neigh	vector of 4 elements containing the ranks of neighbouring processes, -1 indicates a boundary in this direction
 * @param[in] color	red/black coloring of subdomains (0/1)
 * @param[in] icomm	communicator
*/

void AccuDiag(const int nnode, const int id[], const int ik[],const double sk[], double d[], 
	const int nx, const int ny, const int neigh[], const int color, const MPI::Intracomm& icomm);
    
#endif
