
#pragma offload_attribute (push,target(mic))
#include <iostream>
#include <omp.h>                // OpenMP
#pragma offload_attribute (pop)

#include <sstream>
#include <cstdlib>
#include <cstring>
#include <cmath>
#include <offload.h> // to define _Offload_number_of_devices() and _Offload_get_device_number()

// #include <ctime>
#include "mylib.h"

//--------------------------------------------------------------------------
__attribute__ ((target(mic)))
void testThreadCount();

__attribute__ ((target(mic)))
std::string get_device_name();
//--------------------------------------------------------------------------

using namespace std;
// export  OMP_NUM_THREADS=2

int main(int argc, char **argv)
{
    cout << "Hello World " << endl;

//   const int NLOOPS=50;          // chose a value such that the benchmark runs at least 10 sec.
//   unsigned int M=10000, N=10000;
  //const int NLOOPS=20;          // chose a value such that the benchmark runs at least 10 sec.
  const int NLOOPS=2;          // chose a value such that the benchmark runs at least 10 sec.
  //unsigned int M=20000, N=20000;
  unsigned int M=2000, N=2000;
//##########################################################################
 int th_id,nthreads;                // OpenMP
#pragma omp parallel private(th_id)
 {
  th_id = omp_get_thread_num();                 // OpenMP
  stringstream ss;
  ss << "C++: Hello World from thread " << th_id << endl;
  cout << ss.str();
  nthreads = omp_get_num_threads();                // OpenMP
 }
testThreadCount();

  cout << "\nN = " << N << endl;

//  Memory allocation
  cout << "Memory allocation\n";

//##########################################################################
//                          align  x and y
  double *b,*x, *A, *bb;
  A = new double [M*N];
  x = new double [N];
  b = new double [M];
  bb = new double [M];

  cout.precision(2);
  cout << static_cast<double>(N*M+M+N)*sizeof(x[0])/1024/1024/1024 << " GiByte Memory allocated\n";
  cout.precision(6);

//##########################################################################
//  Data initialization
//  Special:  x_i = i+1;  y_i = 1/x_i  ==> <x,y> == N
  for (unsigned int i=0; i<M; ++i)
  {
    for (unsigned int j=0; j<N; ++j)
    {
      A[i*N+j] = ((i+2*j) % 219)+1;
    }
  }

  for (unsigned int j=0; j<N; ++j)
    {
      x[j] = 1.0/((j % 219)+1);
    }


//##########################################################################
  cout << "\nStart Benchmarking\n";

  double tstart, t1, t2;  // timer
#pragma offload target(mic:0) in(b[0:M],A[0:M*N],x[0:N]) out(b[0:M])
{
    testThreadCount();

   matmul(b,A,M,N,x);
// Do calculation
  tstart = omp_get_wtime();                       // OpenMP

  for (int i=0; i<NLOOPS; ++i)
  {
       matmul(b,A,N,M,x);
  }

  t1 = omp_get_wtime() - tstart;                 // OpenMP
}
  t1 /= NLOOPS;           // divide by number of function calls

  //##########################################################################
  //cout << "\nStart Benchmarking BLAS\n";
#pragma offload target(mic:0) in(bb[0:M],A[0:M*N],x[0:N]) out(b[0:M])
{
  matmul_blas(bb,A,M,N,x);
// Do calculation
  tstart = omp_get_wtime();                       // OpenMP

  for (int i=0; i<NLOOPS; ++i)
  {
    matmul_blas(bb,A,M,N,x);
  }

  t2 = omp_get_wtime() - tstart;                 // OpenMP
  t2 /= NLOOPS;           // divide by number of function calls
}


//##########################################################################
// Check the correct result
  for (unsigned int i=0; i<N; ++i)
  {
//    cout << b[i] << "  " << bb[i]<< endl;
    if (fabs(b[i]-bb[i]) > 1e-6)
      { cout << "  !!   W R O N G  result   !!\n";  break; }
  }
  cout << endl;

//##########################################################################
// Timings  and Performance
 cout << endl;
 cout.precision(2);
 cout << "Timing in sec. : " << t1 << endl;
 cout << "GFLOPS         : " << 1e-9*2*N*M/t1 << endl;
 cout << "GiByte/s       : " << static_cast<double>(N*M+M+N)/t1/1024/1024/1024*sizeof(x[0]) << endl;

// Timings  and Performance BLAS
 cout << endl << "BLAS " << endl;
 cout.precision(2);
 cout << "Timing in sec. : " << t2 << endl;
 cout << "GFLOPS         : " << 1e-9*2*N*M/t2 << endl;
 cout << "GiByte/s       : " << static_cast<double>(N*M+M+N)/t2/1024/1024/1024*sizeof(x[0]) << endl;


//##########################################################################
//  Free allocated memory
  delete [] b; delete [] x; delete [] A; delete [] bb;


  return 0;
}

// --------------------------------------------------------------------

std::string get_device_name()
{
 #ifdef __MIC__
  std::string device_name = "MIC ";
#else
  std::string device_name = "HOST";
#endif // __MIC__

 return device_name;
}


void testThreadCount()
{
  int mic_rank = _Offload_get_device_number();
  int num_mics = _Offload_number_of_devices();

   #pragma omp parallel
   {
     int tid = omp_get_thread_num();
     #pragma omp critical
     {
       std::cout << "Hello by thread id: " << tid << " --> " << get_device_name() << ": " << mic_rank << " / " << num_mics << std::endl;
     }
     #pragma omp barrier
     #pragma omp single
     {
       int thread_count = omp_get_num_threads();
       #pragma omp critical
       {
         std::cout << "Thread count: " << thread_count << std::endl;
         std::cout << "======================================" << std::endl;
       }
     }
  }
}
