# Uses MPI from the system with the PGI compilers
#  requires setting of COMPILER=MPI_PGI_

#PGI_PATH =  /opt/pgi/linux86-64/2016/bin/
MPI_DIR = 
ifeq "$(HOSTNAME)" "mephisto.uni-graz.at"
  # mephisto
  PGI_PATH =  /share/apps/pgi/linux86-64/2016/bin/
  MPIDIR   = /usr/mpi/gcc/openmpi-1.8.4/bin/
endif

# laptop until 2014
#OPTIRUN = optirun               ##  now:  export OPTIRUN = "optirun -v"

#MPI_CFLAGS=`mpicxx -showme:compile |sed 's/-pthread//g'`
# http://wiki.seas.harvard.edu/geos-chem/index.php/GEOS-Chem_HP_Compiling
MPI_CFLAGS= $(shell mpicxx -showme:compile |sed 's/-pthread//g')
MPI_LFLAGS= $(shell mpicxx -showme:link |sed 's/-pthread//g')

CC  = ${MPIDIR}mpicc
CXX = ${MPIDIR}mpic++
F77 = ${MPIDIR}mpif77
ifndef LINKER
  LINKER  = ${CC}
endif
LINKER  = ${CXX}

WARNINGS = -Minform=warn

PGI_PROFILING += -Minfo=loop,vect,opt,intensity,mp,accel

#PGI_PROFILING += -Mprof=lines –Minfo=ccff
PGI_PROFILING +=  –Minfo=ccff

#CXXFLAGS +=  -e3 -fast -fastsse  ${PGI_PROFILING} ${WARNINGS} -Mnodepchk
CXXFLAGS +=  -fast -fastsse  ${PGI_PROFILING} ${WARNINGS} -Mnodepchk
CFLAGS   +=  -fast ${PGI_PROFILING} ${WARNINGS} -Mnodepchk
#
#  for OpenACC
# Target architecture (nvidia,host)
# TA_ARCH = nvidia,host
TA_ARCH = -ta=nvidia:cc2+,cuda7.0,fastmath
TA_ARCH = -acc -DNDEBUG -ta=nvidia:cc2+,cuda7.0,fastmath,keepgpu

#,keepgpu
# CFLAGS = -O3 -ta=$(TA_ARCH)
CFLAGS   += -B -gopt $(TA_ARCH)
CXXFLAGS += -B -gopt $(TA_ARCH)
#  -Minfo=all

# libcudart.a is needed for direct CUDA calls
LINKFLAGS  = -gopt $(TA_ARCH) -L${BINDIR}../lib $(PGI_PROFILING)
# -lcudart
# use "pgc++" as compiler in mpicxx
export OMPI_CXXFLAGS=${CXXFLAGS}
#export OMPI_CXXFLAGS= `${MPI_ROOT}/bin/mpicxx -showme:compile |sed 's/-pthread//g'`
export OMPI_CFLAGS=${MPI_CFLAGS} ${CXXFLAGS} 
#`mpicxx -showme:compile |sed 's/-pthread//g'`
export OMPI_LDFLAGS=${MPI_LFLAGS} ${LINKFLAGS} 
export OMPI_LDFLAGS=-Wl,-rpath -Wl,/usr/mpi/gcc/openmpi-1.8.4/lib64 -Wl,--enable-new-dtags -L/usr/mpi/gcc/openmpi-1.8.4/lib64 -lmpi -gopt $(TA_ARCH) -L${BINDIR}../lib $(PGI_PROFILING)
export OMPI_LDFLAGS=-Wl,-rpath -Wl,/usr/mpi/gcc/openmpi-1.8.4/lib64 -Wl,--enable-new-dtags -L/usr/mpi/gcc/openmpi-1.8.4/lib64 -lmpi_cxx -lmpi

#export OMPI_LIBS=
# use "pgc++" as compiler in mpicxx
export OMPI_CXX=pgc++
export OMPI_CC=pgcc

default:	${PROGRAM}

${PROGRAM}:	${OBJECTS}
#	$(LINKER)  $^  ${LINKFLAGS} -o $@
	pgc++  $^  ${OMPI_LDFLAGS} ${LINKFLAGS}  -o $@
#	pgcc  $^  ${OMPI_LDFLAGS} -o $@

clean:
	rm -f ${PROGRAM} ${OBJECTS} *.gpu

clean_all:: clean
	@rm -f *_ *~ *.bak *.log *.out *.tar

run: clean ${PROGRAM}
	${MPIDIR}mpirun -np 4 ${OPTIRUN} ./${PROGRAM}

# tar the current directory
MY_DIR = `basename ${PWD}`
tar: clean_all
	@echo "Tar the directory: " ${MY_DIR}
	@cd .. ;\
	tar cf ${MY_DIR}.tar ${MY_DIR} *default.mk ;\
	cd ${MY_DIR}
# 	tar cf `basename ${PWD}`.tar *

doc:
	doxygen Doxyfile

#########################################################################

.cpp.o:
	$(CXX) -c $(CXXFLAGS) $<

.c.o:
	$(CC) -c $(CFLAGS) $<

.f.o:
	$(F77) -c $(FFLAGS) $<

##################################################################################################
# #    some tools
# #  Simple run time profiling of your code
# #  CXXFLAGS += -g -pg
# #  LINKFLAGS += -pg


# Profiling options PGI, see: pgcollect -help
CPU_PROF = -allcache
GPU_PROF = -cuda=gmem,branch,cc13 -cudainit
#GPU_PROF = -cuda=branch:cc20
#
PROF_FILE = pgprof.out

prof: ${PROGRAM}
#	./$^
#	$(CUDA_HOME)/bin/nvvp &
#	export LD_LIBRARY_PATH=/state/partition1/apps/pgi/linux86-64/12.9/lib:$LD_LIBRARY_PATH
	${OPTIRUN} ${BINDIR}pgcollect $(GPU_PROF) ./$^
	${OPTIRUN} ${BINDIR}pgprof -exe ./$^  $(PROF_FILE) &


# Memory checker (slooooow!!!):
# see doc at /usr/local/cuda/doc/cuda-memcheck.pdf
# mem: ${PROGRAM}
# 	$(CUDA_HOME)memcheck ./$^
