# Basic Defintions for using INTEL compiler suite sequentially
# requires setting of COMPILER=PGI_

# Details for run time information
# export PGI_ACC_TIME=1
# unset PGI_ACC_TIME
# export PGI_ACC_NOTIFY=1
# export PGI_ACC_NOTIFY=3
# unset PGI_ACC_NOTIFY


# mephisto
# my sony
               ##  now:  export OPTIRUN = "optirun -v"

CC  = pgcc
CXX = pgc++
F77 = pgfortran
ifndef LINKER
  LINKER  = ${CC}
endif
LINKER  = ${CXX}


WARNINGS = -Minform=warn

#PGI_PROFILING += -Minfo=loop,vect,opt,intensity,mp,accel
#PGI_PROFILING += -Minfo=accel,mp,ccff -Mautoinline
#PGI_PROFILING += -Minfo=ccff,accel,ipa,loop,lre,mp,opt,par,unified,vect,intensity
PGI_PROFILING += -Minfo=ccff,accel

# -Minfo
# -Mprof=lines
# -Minfo=accel,ccff
# -Mipa=inline

# http://stackoverflow.com/questions/9282461/icpc-c11-with-gcc-stdlib
CXXFLAGS +=  -O3 -e3 -fast -fastsse ${PGI_PROFILING} ${WARNINGS}
CFLAGS   +=  -e3 -fast -fastsse -Msafeptr=all ${PGI_PROFILING} ${WARNINGS}
LINKFLAGS +=
#-lcblas
#PGI_PROFILING += -Mprof=lines
#-Mpfi -Mpfo
#-Minfo
#
#  for OpenACC
# Target architecture (nvidia,host)
#TA_ARCH = -DNDEBUG -ta=nvidia,host
#TA_ARCH = -acc -ta=host
#TA_ARCH = -acc -DNDEBUG -ta=nvidia:cc30,cuda5.5,keepgpu,fastmath,time
#TA_ARCH = -acc -DNDEBUG -ta=nvidia:cc30,cuda5.5,keepgpu,fastmath,time
#TA_ARCH = -ta=nvidia:cc2+,cuda7.0,fastmath,keep,time -acc -DNDEBUG
TA_ARCH = -ta=nvidia:cc2+,fastmath,keep,time -acc -DNDEBUG
#my desktop
#TA_ARCH = -ta=nvidia:maxwell,fastmath,keepgpu,keepptx,lineinfo,loadcache:L2,time -acc -DNDEBUG


#CXXFLAGS += -B -gopt $(TA_ARCH)
CXXFLAGS += $(TA_ARCH)
CFLAGS   += -B -gopt $(TA_ARCH)
#  -Minfo=al

# libcudart.a is needed for direct CUDA calls
LINKFLAGS  += -gopt $(TA_ARCH) -L${BINDIR}../lib $(PGI_PROFILING)
# -lcudart

default:	${PROGRAM}

${PROGRAM}:	${OBJECTS}
	$(LINKER)  $^  ${LINKFLAGS} -o $@

clean:
	rm -f ${PROGRAM} ${OBJECTS} *.gpu *.ptx *.bin

clean_all:: clean
	@rm -f *_ *~ *.bak *.log *.out *.tar

run: clean ${PROGRAM}
	${OPTIRUN} ./${PROGRAM}

# tar the current directory
MY_DIR = `basename ${PWD}`
tar: clean_all
	@echo "Tar the directory: " ${MY_DIR}
	@cd .. ;\
	tar cf ${MY_DIR}.tar ${MY_DIR} *default.mk ;\
	cd ${MY_DIR}
# 	tar cf `basename ${PWD}`.tar *

doc:
	doxygen Doxyfile

#########################################################################

.cpp.o:
	$(CXX) -c $(CXXFLAGS) $<

.c.o:
	$(CC) -c $(CFLAGS) $<

.f.o:
	$(F77) -c $(FFLAGS) $<

##################################################################################################
# #    some tools
# #  Simple run time profiling of your code
# #  CXXFLAGS += -g -Minfo=ccff
# #  LINKFLAGS +=                       # no flag -pg


# Profiling options PGI, see: pgcollect -help
CPU_PROF = -allcache
GPU_PROF = -cuda=gmem,branch,cc2x -cudainit
#GPU_PROF = -cuda=branch:cc20
#
PROF_FILE = pgprof.out

prof: ${PROGRAM}
#	./$^
#	$(CUDA_HOME)/bin/nvvp &
	${OPTIRUN} ${BINDIR}pgprof -f -o $(PROF_FILE) ./$^
	${OPTIRUN} ${BINDIR}pgprof --openacc-profiling on  -i $(PROF_FILE) 2>&1 |more
#	${OPTIRUN} ${BINDIR}pgprof --openacc-profiling on --cpu-profiling-show-ccff on -i $(PROF_FILE)


# Memory checker (slooooow!!!):
# see doc at /usr/local/cuda/doc/cuda-memcheck.pdf
# mem: ${PROGRAM}
# 	$(CUDA_HOME)memcheck ./$^


info:
	pgcpuid
	pgaccelinfo -v
	nvidia-smi
