#ifndef VDOP_FILE
#define VDOP_FILE


/** @brief  Element-wise vector divison x = y/z.
 *  A stride @p iy =0 for the source stride adds a constant value stored in @p y[0].
 *  The same holds for vector @p z.
 *
 * @param[in]  n  number of elements to divide
 * @param[out] x  target vector
 * @param[in]  ix stride for target vector @p x
 * @param[in]  y  source vector
 * @param[in]  iy stride for source vector @p y
 * @param[in]  z  source vector
 * @param[in]  iz stride for source vector @p z
 *
 */
void vddiv(const int n,  float x[], const int ix,
                   const float y[], const int iy,
                   const float z[], const int iz);

/** @brief  Element-wise vector multiplication x = y*z.
 *  A stride @p iy =0 for the source stride adds a constant value stored in @p y[0].
 *  The same holds for vector @p z.
 *
 * @param[in]  n  number of elements to multiply
 * @param[out] x  target vector
 * @param[in]  ix stride for target vector @p x
 * @param[in]  y  source vector
 * @param[in]  iy stride for source vector @p y
 * @param[in]  z  source vector
 * @param[in]  iz stride for source vector @p z
 *
 */

//#pragma acc routine gang
void vdaxpy(const int n, float x[], const float y[],
                   const float a,   const float z[] );


/** @brief  Calculates the Euclidian inner product of two vectors.
 *
 * @param[in]  n  number of elements in both vectors.
 * @param[in]  x source vector
 * @param[in]  y source vector
 * @return Euclidian inner product @f$\langle x,y \rangle@f$
 *
 */
//#pragma acc routine gang
float dscapr(const int n, const float x[], const float y[]);



#endif
 
