#include <iostream>
#include <sstream>              // stringstream

#include <cstdlib>
#include <cstring>
// #include <cmath>
#include <ctime>
#include <omp.h>                // OpenMP  // omp_get_wtime()
#ifdef _OPENACC
#include <accel.h>              // OpenACC
#endif
#include "mylib.h"
using namespace std;

// export  OMP_NUM_THREADS=2

int main(int argc, char **argv)
{
  const int NLOOPS=100;          // chose a value such that the benchmark runs at least 10 sec.
  unsigned int N=10000000;

  //omp_set_num_threads(1);

  int th_id,nthreads;                // OpenMP
#pragma omp parallel private(th_id)
 {
  th_id = omp_get_thread_num();                 // OpenMP
  stringstream ss;
  ss << "C++: Hello World from thread " << th_id << endl;
  cout << ss.str();
  nthreads = omp_get_num_threads();                // OpenMP
 }
// #########################################################################
#ifdef _OPENACC
    acc_init(acc_device_nvidia);             // OpenACC call
    cout << "\nOpenACC Init done.\n";
#endif
// #########################################################################

  cout << "\nN = " << N << endl;

//  Memory allocation
  cout << "Memory allocation\n";

//##########################################################################
//                          align  x and y
  double *x,*y;
  x = new double [N];
  y = new double [N];

  cout.precision(2);
  cout << 2.0*N*sizeof(x[0])/1024/1024/1024 << " GiByte Memory allocated\n";
  cout.precision(6);

//##########################################################################
//  Data initialization
//  Special:  x_i = i+1;  y_i = 1/x_i  ==> <x,y> == N
  for (unsigned int i=0; i<N; ++i)
  {
    x[i] = (i % 219)+1;
    y[i] = 1.0/x[i];
  }

//##########################################################################
  cout << "\nStart Benchmarking\n";
  double tstart, t1;  // timer
  double sk(0.0);
#pragma acc data copyin (x[0:N],y[0:N])
{                                      // Parantheses are important!!
// Do calculation
  //tstart = omp_get_wtime();                       // OpenMP
  tstart = clock();

for (int i=0; i<NLOOPS; ++i)
  {
     sk = scalar(N,x,y);
    //sk = scalar_blas(N,x,y);
  }

  //t1 = omp_get_wtime() - tstart;                 // OpenMP
  t1 = (clock() - tstart) / CLOCKS_PER_SEC;
  t1 /= NLOOPS;           // divide by number of function calls
} // end: #pragma acc data
//##########################################################################
// Check the correct result
  cout << "\n <x,y> = " << sk << endl;
  if (static_cast<int>(sk) != N)
  {
     cout << "  !!   W R O N G  result   !!\n";
  }
  cout << endl;

//##########################################################################
// Timings  and Performance
 cout << endl;
 cout.precision(2);
 cout << "Timing in sec. : " << t1 << endl;
 cout << "GFLOPS         : " << 2.0e-9*N/t1 << endl;
 cout << "GiByte/s       : " << 2.0*N/t1/1024/1024/1024*sizeof(x[0]) << endl;

//##########################################################################
//  Free allocated memory
  delete [] y; delete [] x;

  return 0;
}
