#ifndef JACSOLVE_FILE
#define JACSOLVE_FILE

/**
 * Solves linear system of equations  K @p u = @p f  via the Jacobi iteration.
 * We use a distributed symmetric  CSR matrix K(@p sk,@p id, @p ik) and initial guess of the 
 * solution is set to 0.
 * @param[in] nnode	number of nodes and number of rows
 * @param[in] sk	vector non-zero entries of CSR matrix
 * @param[in] id	index vector containing the first entry in a CSR row
 * @param[in] ik	column index vector of CSR matrix
 * @param[in] f		distributed local vector storing the right hand side
 * @param[out] u	accumulated local vector storing the solution.
*/

void JacobiSolve(const int nnode,
//  const int nx, const int ny,
// 	const int neigh[], const int color,
// 	const MPI_Comm& icomm,
	const int id[], const int ik[], const double sk[],
	const double f[], double u[] );


#endif
