#include <iostream>
#include <cmath>
using namespace std;
#include "vdop.h"
#include "getmatrix.h"
#include "jacsolve.h"

void JacobiSolve(const int nnode,
// 	const int neigh[], const int color,
// 	const MPI::Intracomm& icomm,
	const int id[], const int ik[], const double sk[],
	const double f[], double u[] )
{
  const double omega   = 1.0;
  const int    maxiter = 1000;
  const double tol  = 1e-5,                  /* tolerance   */
               tol2 = tol*tol;               /* tolerance^2 */

  double *dd = new double [nnode],           /* matrix diagonal */
         *r  = new double [nnode],           /* residual */
         *w  = new double [nnode];           /* correction */
  cout << endl << " Start Jacobi solver for " << nnode << " d.o.f.s"  << endl;
  /*  Choose initial guess  */
  for (int k=0; k<nnode; ++k) { u[k] = 0.0; }  /*  u := 0         */


  GetDiag(nnode, id, ik, sk, dd);            /*  diag */
    //DebugVector(nnode,dd);{int ijk; cin >> ijk;}

  /*  Initial sweep  */
  Defect(r, f, u, nnode, id, ik, sk);        /*  r := f - K*u   */

  vddiv(nnode, w, 1, r, 1, dd, 1);           /*  w := D^{-1}*r  */
  double sigma0 = dscapr(nnode,w,r);         /* s0 := <w,r>     */

  /* Iteration sweeps */
  int iter  = 0;
  double sigma = sigma0;
  while ( sigma > tol2*sigma0 && maxiter>iter)
  {
    ++iter;
    vdaxpy(nnode, u, u, omega, w );          /*  u := u + om*w  */
    Defect(r, f, u, nnode, id, ik, sk);      /*  r := f - K*u   */
    vddiv(nnode, w, 1, r, 1, dd, 1);         /*  w := D^{-1}*r  */
    sigma = dscapr(nnode,w,r);               /* s0 := <w,r>     */
//      	cout << "Iteration " << iter << " : " << sqrt(sigma/sigma0) << endl;
  }
  cout << "aver. Jacobi rate :  " << exp(log(sqrt(sigma/sigma0))/iter) << "  (" << iter << " iter)"<< endl;
  cout << "final error: " << sqrt(sigma/sigma0) << " (rel)   " << sqrt(sigma) << " (abs)\n";

  delete [] w; delete [] r; delete [] dd;
  return;
}
