#include <iostream>
#include <sstream>

#include <cstdlib>
#include <cstring>
#include <cmath>
#include <ctime>
#include "mylib.h"
using namespace std;

int main(int argc, char **argv)
{
//   const int NLOOPS=50;          // chose a value such that the benchmark runs at least 10 sec.
//   unsigned int M=10000, N=10000;
  const int NLOOPS=50;          // chose a value such that the benchmark runs at least 10 sec.
  unsigned int M=10000, N=10000;
//##########################################################################
//  Memory allocation
  cout << "Memory allocation\n";

//##########################################################################
//                          align  x and y
  double *b,*x, *A, *bb;
  A = new double [M*N];
  x = new double [N];
  b = new double [M];
  bb = new double [M];

  cout.precision(2);
  cout << static_cast<double>(N*M+M+N)*sizeof(x[0])/1024/1024/1024 << " GiByte Memory allocated\n";
  cout.precision(6);

//##########################################################################
//  Data initialization
//  Special:  x_i = i+1;  y_i = 1/x_i  ==> <x,y> == N
  for (unsigned int i=0; i<M; ++i)
  {
    double s(0.0);
    for (unsigned int j=0; j<N; ++j)
    {
      A[i*N+j] = ((i+2*j) % 219)+1;
    }
  }

  for (unsigned int j=0; j<N; ++j)
    {
      x[j] = 1.0/((j % 219)+1);
    }


//##########################################################################
  cout << "\nStart Benchmarking\n";

  double tstart, t1, t2;  // timer

   matmul(b,A,M,N,x);
// Do calculation
  tstart = clock();       // start timer

  for (int i=0; i<NLOOPS; ++i)
  {
       matmul(b,A,N,M,x);
  }

  t1 = clock()-tstart;    // stop timer
  t1 /= CLOCKS_PER_SEC;   // now, t1 in seconds
  t1 /= NLOOPS;           // divide by number of function calls

  //##########################################################################
  cout << "\nStart Benchmarking BLAS\n";

  matmul_blas(bb,A,M,N,x);
// Do calculation
  tstart = clock();       // start timer

  for (int i=0; i<NLOOPS; ++i)
  {
    matmul_blas(bb,A,M,N,x);
  }

  t2 = clock()-tstart;    // stop timer
  t2 /= CLOCKS_PER_SEC;   // now, t1 in seconds
  t2 /= NLOOPS;           // divide by number of function calls


//##########################################################################
// Check the correct result
//   for (unsigned int i=0; i<N; ++i)
//   {
// //    cout << b[i] << "  " << bb[i]<< endl;
//     if (fabs(b[i]-bb[i]) > 1e-6)
//       { cout << "  !!   W R O N G  result   !!\n";  break; }
//   }
//   cout << endl;

//##########################################################################
// Timings  and Performance
 cout << endl;
 cout.precision(2);
 cout << "Timing in sec. : " << t1 << endl;
 cout << "GFLOPS         : " << 1e-9*2*N*M/t1 << endl;
 cout << "GiByte/s       : " << static_cast<double>(N*M+M+N)/t1/1024/1024/1024*sizeof(x[0]) << endl;

// Timings  and Performance BLAS
 cout << endl << "BLAS " << endl;
 cout.precision(2);
 cout << "Timing in sec. : " << t2 << endl;
 cout << "GFLOPS         : " << 1e-9*2*N*M/t2 << endl;
 cout << "GiByte/s       : " << static_cast<double>(N*M+M+N)/t2/1024/1024/1024*sizeof(x[0]) << endl;


//##########################################################################
//  Free allocated memory
  delete [] b; delete [] x; delete [] A; delete [] bb;

  return 0;
}
