//    template for Master-HPC WS10

#include <iostream>
#include <sstream>

#include <cstdlib>
#include <cstring>
// #include <cmath>
#include <ctime>
#include "mylib.h"
using namespace std;

int main(int argc, char **argv)
{
  const int NLOOPS=10;          // chose a value such that the benchmark runs at least 10 sec.
//   unsigned int N=10000000;
  unsigned int N=10000000;
//##########################################################################
//  allocate memory for x,y

  double *x = new double [N];
  double *y = new double [N];

  cout.precision(2);
  cout << 2.0*N*sizeof(x[0])/1024/1024/1024 << " GiByte Memory allocated\n";
  cout.precision(6);

//##########################################################################
//  Data initialization
//  Special:  x_i = i+1;  y_i = 1/x_i  ==> <x,y> == N
  for (unsigned int i=0; i<N; ++i)
  {
    x[i] = (i % 219)+1;
    y[i] = 1.0/x[i];
  }

//##########################################################################
  cout << "\nStart Benchmarking\n";

  double tstart, t1;  // timer
  double sk(0.0);

// Do calculation
  tstart = clock();       // start timer

  for (int i=0; i<NLOOPS; ++i)
  {
     sk = scalar(N,x,y);
  }

  t1 = clock()-tstart;    // stop timer
  t1 /= CLOCKS_PER_SEC;   // now, t1 in seconds
  t1 /= NLOOPS;           // divide by number of function calls

//##########################################################################
// Check the correct result
  cout << "\n <x,y> = " << sk << endl;
  if (static_cast<int>(sk) != N)
  {
     cout << "  !!   W R O N G  result   !!\n";
  }
  cout << endl;

//##########################################################################
// Timings  and Performance
 cout << endl;
 cout.precision(2);
 cout << "Timing in sec. : " << t1 << endl;
 cout << "GFLOPS         : " << 2.0e-9*N/t1 << endl;
 cout << "GiByte/s       : " << 2.0*N/t1/1024/1024/1024*sizeof(x[0]) << endl;

//##########################################################################
//  Free allocated memory
  delete [] y; delete [] x;

  return 0;
}
