# Basic Defintions for using GNU-compiler suite sequentially
# requires setting of COMPILER=CLANG_

# Additional packes for OPenMP with clang:  libomp5   libomp-dev
# see    http://packages.ubuntu.com/de/source/xenial/misc/openmprtl

CC	= clang
CXX     = clang++
#CXX     = clang++-3.8
#F77	= gfortran
LINKER  = ${CXX}

# on mephisto:
CXXFLAGS  += -I/share/apps/atlas/include
LINKFLAGS += -L/share/apps/atlas/lib
LINKFLAGS   += -lcblas -latlas
# Der <cblas.h> Header muss mit extern "C" versehen werden, damit g++ alles findet.

#http://clang.llvm.org/docs/UsersManual.html#options-to-control-error-and-warning-messages
WARNINGS = -Weverything -Wno-c++98-compat -ferror-limit=3
#-fsyntax-only -Wdocumentation -Wconversion -Wshadow -Wfloat-conversion -pedantic
CXXFLAGS += -Ofast -std=c++11 ${WARNINGS}
# -ftrapv

# OpenMP : compiles, links, but runs only with one thread
#           http://stackoverflow.com/questions/33357029/using-openmp-with-clang
#       -fopenmp-use-tl  #file:///usr/share/doc/clang-3.8-doc/html/UsersManual.html#openmp-features
CXXFLAGS += -fopenmp
#-I/usr/lib/gcc/x86_64-linux-gnu/5/include -Wno-reserved-id-macro -Wno-deprecated
LINKFLAGS += -fopenmp
#CXXFLAGS +=  -fopenmp=libgomp -I/usr/lib/gcc/x86_64-linux-gnu/5/include/  -Wno-reserved-id-macro -Wno-deprecated
#LINKFLAGS += -fopenmp=libgomp -L/usr/lib/gcc/x86_64-linux-gnu/5
#-fopenmp=libomp          # http://stackoverflow.com/questions/33400462/omp-h-file-not-found-when-compiling-using-clang


default: ${PROGRAM}

${PROGRAM}:	${OBJECTS}
	$(LINKER)  $^  ${LINKFLAGS} -o $@

clean:
	@rm -f ${PROGRAM} ${OBJECTS}

clean_all:: clean
	@rm -f *_ *~ *.bak *.log *.out *.tar

run: clean ${PROGRAM}
#	time  ./${PROGRAM}
	./${PROGRAM}

# tar the current directory
MY_DIR = `basename ${PWD}`
tar: clean_all
	@echo "Tar the directory: " ${MY_DIR}
	@cd .. ;\
	tar cf ${MY_DIR}.tar ${MY_DIR} *default.mk ;\
	cd ${MY_DIR}
# 	tar cf `basename ${PWD}`.tar *

doc:
	doxygen Doxyfile

#########################################################################

.cpp.o:
	$(CXX) -c $(CXXFLAGS) $<

.c.o:
	$(CC) -c $(CFLAGS) $<

.f.o:
	$(F77) -c $(FFLAGS) $<

##################################################################################################
#    some tools
# Cache behaviour (CXXFLAGS += -g  tracks down to source lines; no -pg in linkflags)
cache: ${PROGRAM}
	valgrind --tool=callgrind --simulate-cache=yes ./$^
#	kcachegrind callgrind.out.<pid> &
	kcachegrind `ls -1tr  callgrind.out.* |tail -1`

# Check for wrong memory accesses, memory leaks, ...
# use smaller data sets
mem: ${PROGRAM}
	valgrind -v --leak-check=yes --tool=memcheck --undef-value-errors=yes --track-origins=yes --log-file=$^.addr.out --show-reachable=yes ./$^

#  Simple run time profiling of your code
#  CXXFLAGS += -g -pg
#  LINKFLAGS += -pg
prof: ${PROGRAM}
	./$^
	gprof -b ./$^ > gp.out
#	kprof -f gp.out -p gprof &
