#include <iostream>
#include <cmath>
#include <omp.h>                // OpenMP
using namespace std;
#include "vdop.h"
#include "getmatrix.h"
#include "jacsolve.h"

void JacobiSolve(const int nnode,
// 	const int neigh[], const int color,
// 	const MPI::Intracomm& icomm,
                 const int id[], const int ik[], const double sk[],
                 const double f[], double u[] )
{
    const double omega   = 1.0;
    const int    maxiter = 1000;
    const double tol  = 1e-5,                  /* tolerance   */
                 tol2 = tol * tol;             /* tolerance^2 */

    double *dd = new double [nnode],           /* matrix diagonal */
    *r  = new double [nnode],           /* residual */
    *w  = new double [nnode];           /* correction */
    //#pragma omp parallel
    {
        //#pragma omp barrier
        //#pragma omp master
        {
            cout << endl << " Start Jacobi solver for " << nnode << " d.o.f.s"  << endl;
        }

        GetDiag(nnode, id, ik, sk, dd);            /*  diag */

        /*  Choose initial guess  */
        #pragma omp parallel for
        for (int k=0; k<nnode; ++k) { u[k] = 0.0; }  /*  u := 0         */

        /*  Initial sweep  */
        Defect(r, f, u, nnode, id, ik, sk);        /*  r := f - K*u   */

        #pragma omp parallel for 
        for (int k=0; k<nnode; ++k) { w[k] = r[k]/dd[k]; }  /*  w := D^{-1}*r  */

        int iter  = 0;
        double sigma0 = dscapr(nnode, w, r);              /* s0 := <w,r>     */
        double sigma  = sigma0;

        /* Iteration sweeps */
        while ( sigma > tol2 * sigma0 && maxiter > iter) {
            ++iter;
            vdaxpy(nnode, u, u, omega, w );          /*  u := u + om*w  */
            Defect(r, f, u, nnode, id, ik, sk);      /*  r := f - K*u   */

            #pragma ivdep
            #pragma omp parallel for
            for (int k=0; k<nnode; ++k) { w[k] = r[k]/dd[k]; }  /*  w := D^{-1}*r  */
            
            sigma = dscapr(nnode, w, r);             /* s0 := <w,r>     */

      	    //if ( omp_get_thread_num()==0) cout << "Iteration " << iter << " : " << sqrt(sigma/sigma0) << endl;
            //#pragma omp barrier
        }
        //#pragma omp barrier
        //#pragma omp master
        {
            cout << "aver. Jacobi rate :  " << exp(log(sqrt(sigma / sigma0)) / iter) << endl;
            cout << "final error: " << sqrt(sigma / sigma0) << " (rel)   " << sqrt(sigma) << " (abs)\n";
        }
    }

    delete [] w;
    delete [] r;
    delete [] dd;
    return;
}

