#include <cmath>
using namespace std;

#ifdef __INTEL_COMPILER 
  #pragma message(" ##########  Use of MKL  ###############")
  #include <mkl.h>                 // MKL
#else
  #pragma message(" ##########  Use of CBLAS  ###############")
extern "C"
{
  #include <cblas.h>               // cBLAS Library
}
#endif

#include <omp.h>                // OpenMP

#include "mylib.h"


double scalar(const unsigned int N, const double x[], const double y[])
{
 double sum = 0.0;
 unsigned int i;
#pragma omp parallel for  default(none) private(i) shared(x,y) schedule(static) reduction(+:sum)
 for (i=0; i<N; ++i)
 {
    sum += x[i]*y[i];
 }
 return sum;
}


double norm(const unsigned int N, const double x[])
{
 double sum = 0.0;
 for (unsigned int i=0; i<N; ++i)
 {
    sum += x[i]*x[i];
 }
 return sqrt(sum);
}

